import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.util.List;


/**
 * Write a description of class Rabbit here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class Rabbit extends Animal
{
    // Characteristics shared by all rabbits (static fields).

    // The age at which a rabbit can start to breed.
    private static final int BREEDING_AGE = 5;
    // The age to which a rabbit can live.
    private static final int MAX_AGE = 50;
    // The likelihood of a rabbit breeding (in percent).
    private static final double BREEDING_PROBABILITY = 12;
    // The maximum number of births.
    private static final int MAX_LITTER_SIZE = 5;
    // The food value of a single corn. In effect, this is the
    // number of steps a rabbit can go before it has to eat again.
    private static final int CORN_FOOD_VALUE = 3;
    
    // Individual characteristics (instance fields).
  // The rabbits's food level, which is increased by eating corn.
    private int foodLevel;

    /**
     * Default constructor for testing.
     */
    public Rabbit()
    {
        this(true);
    }

    /**
     * Create a new rabbit. A rabbit may be created with age
     * zero (a new born) or with a random age.
     * 
     * @param randomAge If true, the rabbit will have a random age.
     */
    public Rabbit(boolean randomAge)
    {
        super();
        if(randomAge) {
            setAge(Greenfoot.getRandomNumber(MAX_AGE));
            foodLevel = Greenfoot.getRandomNumber(CORN_FOOD_VALUE);
        }
        else {
            // leave age at 0
            foodLevel = CORN_FOOD_VALUE;
        }
    }
    
    /**
     * This is what the rabbit does most of the time - it runs 
     * around. Sometimes it will breed or die of old age.
     */
    public void act() 
    {
        incrementAge();
        incrementHunger(); // BIG ADDITION
        if (isAlive()) {
            int births = breed();
            for(int b = 0; b < births; b++) {
                Location loc = getField().freeAdjacentLocation(getX(), getY());
                if (loc != null) {
                    Rabbit newRabbit = new Rabbit(false);
                    getField().addObject(newRabbit, loc.getX(), loc.getY());
                }
            }
             // Move towards the source of food if found.
            Location newLocation = findFood(getX(), getY());
            if(newLocation == null) {  // no food found - move randomly
                newLocation = getField().freeAdjacentLocation(getX(), getY());
            }
            // Only transfer to the updated field if there was a free location
            if(newLocation != null) {
                setLocation(newLocation.getX(), newLocation.getY());
            }
            else {
                // can neither move nor stay - overcrowding - all locations taken
                setDead();
            }
        }
    }    
    
    /**
     * Increase the age.
     * This could result in the rabbit's death.
     */
    private void incrementAge()
    {
        setAge(getAge() + 1);
        if(getAge() > MAX_AGE) {
            setDead();
        }
    }
    
    /**
     * Make this rabbit more hungry. This could result in the rabbit's death.
     */
    private void incrementHunger()
    {
        foodLevel--;
        if(foodLevel <= 0) {
            setDead();
        }
    }
    
    /**
     * Tell the rabbit to look for corn adjacent to its current location.
     * Only the first live corn is eaten.
     * @param field The field in which it must look.
     * @param location Where in the field it is located.
     * @return Where food was found, or null if it wasn't.
     */
    private Location findFood(int x, int y)
    {
        List corn = getNeighbours(1, true, Corn.class);
        // do I need to shuffle?
        if (corn.isEmpty()) {
            return null;
        }
        else {
            Corn cornplant = (Corn) corn.get(0);
            Location loc = new Location(cornplant.getX(), cornplant.getY());
            cornplant.setDead();  // eat it
            foodLevel = CORN_FOOD_VALUE;
            return loc;
        }
    }
    
    /**
     * Generate a number representing the number of births,
     * if it can breed.
     * @return The number of births (may be zero).
     */
    private int breed()
    {
        int births = 0;
        if(canBreed() && Greenfoot.getRandomNumber(100) <= BREEDING_PROBABILITY) {
            births = Greenfoot.getRandomNumber(MAX_LITTER_SIZE) + 1;
        }
        return births;
    }
    
    /**
     * @return A string representation of the rabbit.
     */
    public String toString()
    {
        return "Rabbit, age " + getAge();
    }

    /**
     * A rabbit can breed if it has reached the breeding age.
     */
    private boolean canBreed()
    {
        return getAge() >= BREEDING_AGE;
    }
}
